<?php
/**
Plugin Name: WooCommerce Support for JustLanded
Description: This plugin provides additional features for integration with WooCommerce. Requires JustLanded for WordPress 1.5.0 or higher.
Author: ShapingRain.com Labs
Version: 1.4.1
Author URI: http://www.shapingrain.com
 */

function justlanded_plugin_activation_check(){
	if (! (function_exists('is_woocommerce')) ) {
		deactivate_plugins(basename(__FILE__)); // Deactivate ourself
		wp_die("Sorry, but you can't run this plugin before installing WooCommerce itself. You can find more information in the WooCommerce integration tutorial that ships in the documentation. Contact support@shapingrain.com if you need additional assistance.");
	}
}
register_activation_hook(__FILE__, 'justlanded_plugin_activation_check');



function justlanded_woocommerce_options_page_banner() {
	global $of_options;
	global $of_pages;

	if (!is_admin()) return;

	$of_options[] = array("name" => "Page Banner Content (WooCommerce)",
	                      "id" => "page_banner_woocommerce",
	                      "desc" => "Can contain HTML, shortcodes allowed.",
	                      "descposition" => "heading",
	                      "std" => "<h1>[justlanded_pagetitle]</h1>",
	                      "type" => "textarea");
}


function justlanded_woocommerce_options() {
	global $of_options;

	if (!is_admin()) return;

	$of_options[] = array("name" => "WooCommerce",
	                      "type" => "heading");

	/* Action Buttons */
	$of_options[] = array("name" => "Action Buttons Integration",
	                      "desc" => "Allow for action buttons to add products to the cart.",
	                      "descposition" => "heading",
	                      "type" => "sectionheader");

	$of_products=array();
	$args = array( 'post_type' => 'product', 'posts_per_page' => '-1');
	$h = new WP_Query( $args );
	$products = $h->get_posts();

	if (isset($products) && is_array($products))
	{
		foreach ($products as $product) {
			$of_products[$product->ID] = $product->post_title;
		}
	}
	wp_reset_query();

	$of_options[] = array("name" => "First Button: Select a product",
	                      "desc" => "Select a product for this button. If none is selected, the button's default URL will be used.",
	                      "id" => "action_button1_product",
	                      "std" => "",
	                      "type" => "select",
	                      "storage" => "value",
	                      "options" => $of_products);

	$of_options[] = array("name" => "Second Button: Select a product",
	                      "desc" => "Select a product for this button. If none is selected, the button's default URL will be used.",
	                      "id" => "action_button2_product",
	                      "std" => "",
	                      "type" => "select",
	                      "storage" => "value",
	                      "options" => $of_products);

	/* Pricing Tables */

	$of_options[] = array("name" => "Pricing Table Integration",
	                      "desc" => "Assign products to pricing table buttons",
	                      "descposition" => "heading",
	                      "type" => "sectionheader");

	$data = get_option(OPTIONS);

	if (isset($data['pricing_packages']) && is_array($data['pricing_packages']) && count($data['pricing_packages']) > 0) {
		$x=0;
		foreach($data['pricing_packages'] as $package) {
			if (isset($package['title']) && trim($package['title']) != "") $title = $package['title']; else $title = "[No title]";
			$order = $package['order'];
			$of_options[] = array("name" => "Package ".($x+1).": '".$title."'",
			                      "desc" => "Select a product for this package's action button.",
			                      "id" => "pricing_table_package".$order."_product",
			                      "std" => "",
			                      "type" => "select",
			                      "storage" => "value",
			                      "options" => $of_products);
			$x++;
		}
	}
	else {
		$of_options[] = array("name" => "Pricing Table",
		                      "desc" => "",
		                      "id" => "justlanded_woocommerce_pricing_table_info",
		                      "std" => "<h3 style=\"margin: 0 0 10px;\">No Pricing Table found.</h3>
					In order to use this feature, please create a pricing table for this profile.",
		                      "icon" => true,
		                      "type" => "info");

	}

	/* Redirection */
	$of_options[] = array("name" => "Redirection Target",
	                      "desc" => "Redirect customers after add-to-cart action.",
	                      "descposition" => "heading",
	                      "type" => "sectionheader");

	$of_targets = array(
		"custom" => "Custom URL",
		"woocommerce_cart_page_id" => "Shopping Cart",
		"woocommerce_checkout_page_id" => "Checkout Page",
		"woocommerce_shop_page_id" => "Shop Page",
	);

	$of_options[] = array("name" => "Select A Page",
	                      "desc" => "Select an existing page as redirection target URL.",
	                      "id" => "global_justlanded_wc_redirect_page",
	                      "std" => "woocommerce_cart_page_id",
	                      "global" => true,
	                      "type" => "select",
	                      "storage" => "value",
	                      "options" => $of_targets);

	$of_options[] = array("name" => "Custom URL",
	                      "desc" => "Enter a custom redirection target URL.",
	                      "id" => "global_justlanded_wc_redirect_url",
	                      "global" => true,
	                      "std" => "",
	                      "type" => "text");

	$of_options[] = array("name" => "Empty Cart",
	                      "type" => "sectionheader_small");

	$of_options[] = array("name" => "Empty cart before each action?",
	                      "desc" => "Check this if you want the cart to be emptied before executing an add-to-cart action.",
	                      "id" => "global_justlanded_wc_empty_cart",
	                      "global" => true,
	                      "std" => 0,
	                      "type" => "checkbox");

	/* Display Options */
	$of_options[] = array("name" => "Display Options",
	                      "desc" => "Change the way WooCommerce templates are displayed.",
	                      "descposition" => "heading",
	                      "type" => "sectionheader");

	$of_options[] = array("name" => "WooCommerce Default Profile",
	                      "desc" => "This profile will be used for all WooCommerce pages.",
	                      "descposition" => "heading",
	                      "id" => "global_justlanded_wc_profile",
	                      "std" => "0",
	                      "global" => true,
	                      "type" => "profile",
	                      "source" => 'wc_default',
	                      "experimental" => 1
	);

	$of_options[] = array("name" => "Other Options",
	                      "type" => "sectionheader_small");

	$of_options[] = array("name" => "Display page titles?",
	                      "desc" => "Check this if you want product detail and archive pages to display a page title.",
	                      "id" => "global_justlanded_wc_titles",
	                      "global" => false,
	                      "std" => 1,
	                      "type" => "checkbox");

	$of_options[] = array("name" => "Display sidebar?",
	                      "desc" => "Check this if you want product detail and archive pages to have a sidebar.",
	                      "id" => "global_justlanded_wc_sidebar",
	                      "global" => false,
	                      "std" => 0,
	                      "type" => "checkbox");

	$of_options[] = array("name" => "Display breadcrumb navigation?",
	                      "desc" => "Check this if you want product detail and archive pages to display a breadcrumb navigation bar.",
	                      "id" => "global_justlanded_wc_breadcrumb",
	                      "global" => false,
	                      "std" => 0,
	                      "type" => "checkbox");



}

function justlanded_woocommerce_head() {
	global $data;

	if( is_child_theme() ) {
		$temp_obj = wp_get_theme();
		$theme_obj = wp_get_theme( $temp_obj->get('Template') );
	} else {
		$theme_obj = wp_get_theme();
	}
	$theme_name = $theme_obj->get('Name');
	$empty_cart = intval(get_option($theme_name . '_global_justlanded_wc_empty_cart'));

	// Action Button 1
	if (isset($data['action_button1_product']) && $data['action_button1_product'] != 0) {
		$_pf = new WC_Product_Factory();
		$product = $_pf->get_product( $data['action_button1_product'] );
		if ($product->product_type == "external") {
			$permalink = $product->get_product_url();
		} else {
			$permalink = get_permalink($data['action_button1_product']);
			if (substr_count($permalink, "?") == 0) $permalink .= "?"; else $permalink .= "&";
			$permalink .= "add-to-cart=".$data['action_button1_product'];
			if ($empty_cart == true) {
				if (substr_count($permalink, "?") > 0) $permalink .= "&empty-cart=1"; else $permalink .= "?empty-cart=1";
			}
		}
		$data['action_button1_link'] = $permalink;
	}

	// Action Button 2
	if (isset($data['action_button2_product']) && $data['action_button2_product'] != 0) {
		$_pf = new WC_Product_Factory();
		$product = $_pf->get_product( $data['action_button2_product'] );
		if ($product->product_type == "external") {
			$permalink = $product->get_product_url();
		} else {
			$permalink = get_permalink($data['action_button2_product']);
			if (substr_count($permalink, "?") == 0) $permalink .= "?"; else $permalink .= "&";
			$permalink .= "add-to-cart=".$data['action_button2_product'];
			if ($empty_cart == true) {
				if (substr_count($permalink, "?") > 0) $permalink .= "&empty-cart=1"; else $permalink .= "?empty-cart=1";
			}
		}
		$data['action_button2_link'] = $permalink;
	}

	// Pricing Table
	if (isset($data['pricing_packages']) && is_array($data['pricing_packages']) && count($data['pricing_packages']) > 0) {
		foreach($data['pricing_packages'] as $package) {
			$order = $package['order'];
			if (isset($data['pricing_table_package'. $order .'_product']) && $data['pricing_table_package'. $order .'_product'] != 0) {
				$pid = $data['pricing_table_package'. $order .'_product'];
				$_pf = new WC_Product_Factory();
				$product = $_pf->get_product( $pid );
				if ($product->product_type == "external") {
					$permalink = $product->get_product_url();
				} else {
					$permalink = get_permalink($pid);
					if (substr_count($permalink, "?") == 0) $permalink .= "?"; else $permalink .= "&";
					$permalink .= "add-to-cart=".$pid;
					if ($empty_cart == true) {
						if (substr_count($permalink, "?") > 0) $permalink .= "&empty-cart=1"; else $permalink .= "?empty-cart=1";
					}
				}
				$data['pricing_packages'][$order]['link'] = $permalink;
			}
		}
	}

}

add_filter('woocommerce_add_to_cart_redirect', 'custom_add_to_cart_redirect');
function custom_add_to_cart_redirect() {
	global $data;

	if( is_child_theme() ) {
		$temp_obj = wp_get_theme();
		$theme_obj = wp_get_theme( $temp_obj->get('Template') );
	} else {
		$theme_obj = wp_get_theme();
	}
	$theme_name = $theme_obj->get('Name');

	$url = get_permalink(get_option('woocommerce_cart_page_id'));

	$page = get_option($theme_name . '_global_justlanded_wc_redirect_page');
	$custom_url = get_option($theme_name . '_global_justlanded_wc_redirect_url');


	if ($page == "custom" && $custom_url != "") {
		$url = $custom_url;
	}
	elseif ($page != "") {
		$url = get_permalink(get_option($page));
	}

	return $url;
}

add_action('admin_head', 'justlanded_woocommerce_admin_css');
function justlanded_woocommerce_admin_css() {
	echo '<style>
    #of_container #of-nav li.woocommerce a {
    background: url('.plugin_dir_url("justlanded_woocommerce.php").'justlanded-woocommerce/pricing.png);
    }
  </style>';
}

// add custom CSS for WooCommerce support
add_action('wp_enqueue_scripts', 'justlanded_woocommerce_enqueue_custom_stylesheet');
if(!function_exists('justlanded_woocommerce_enqueue_custom_stylesheet')) {
	function justlanded_woocommerce_enqueue_custom_stylesheet() {
		wp_register_style('justlanded_woocommerce_custom_stylesheet', plugin_dir_url("justlanded_woocommerce.php"). 'justlanded-woocommerce/justlanded_woocommerce.css', array(),'1.0.0', 'all');
		wp_enqueue_style('justlanded_woocommerce_custom_stylesheet');
	}
}

add_action('justlanded_after_dynamic_css', 'justlanded_woocommerce_custom_css');
if(!function_exists('justlanded_woocommerce_custom_css')) {
	function justlanded_woocommerce_custom_css() {
		global $data;
		?>
		<style type="text/css">
			<?php if (isset($data['buttons_base_color_active']) && $data['buttons_base_color_active'] == 1) { ?>
			.woocommerce a.button.alt, .woocommerce button.button.alt, .woocommerce input.button.alt, .woocommerce #respond input#submit.alt, .woocommerce #content input.button.alt, .woocommerce-page a.button.alt, .woocommerce-page button.button.alt, .woocommerce-page input.button.alt, .woocommerce-page #respond input#submit.alt, .woocommerce-page #content input.button.alt {
				background: <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?> !important; /* Old browsers */
				background: url(data:image/svg+xml;base64,<?php echo justlanded_get_svg_gradient(justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']), justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color'])); ?>) !important;
				background: -moz-linear-gradient(top, <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?> 1%, <?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?> 100%) !important; /* FF3.6+ */
				background: -webkit-gradient(linear, left top, left bottom, color-stop(1%, <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?>), color-stop(100%, <?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?>)) !important; /* Chrome,Safari4+ */
				background: -webkit-linear-gradient(top, <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?> 1%, <?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?> 100%) !important; /* Chrome10+,Safari5.1+ */
				background: -o-linear-gradient(top, <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?> 1%, <?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?> 100%) !important; /* Opera 11.10+ */
				background: -ms-linear-gradient(top, <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?> 1%, <?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?> 100%) !important; /* IE10+ */
				background: linear-gradient(to bottom, <?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?> 1%, <?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?> 100%) !important; /* W3C */
				filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='<?php echo justlanded_button_color("buttons_gradient_1/start", "#fea817",$data['buttons_base_color']); ?>', endColorstr='<?php echo justlanded_button_color("buttons_gradient_1/end", "#c54200",$data['buttons_base_color']); ?>', GradientType=0) !important; /* IE6-8 */
				color: <?php echo justlanded_button_color("buttons_text", "#ffffff",$data['buttons_base_color']); ?> !important;
				border-color: <?php echo justlanded_button_color("buttons_newsletter_border1", "#eb9f29",$data['buttons_base_color']); ?> !important;
				text-shadow: 0 -1px 0 <?php echo justlanded_button_color("buttons_text_shadow", "#af5305",$data['buttons_base_color']); ?> !important;
			}
			<?php } else {?>
			.woocommerce a.button.alt, .woocommerce button.button.alt, .woocommerce input.button.alt, .woocommerce #respond input#submit.alt, .woocommerce #content input.button.alt, .woocommerce-page a.button.alt, .woocommerce-page button.button.alt, .woocommerce-page input.button.alt, .woocommerce-page #respond input#submit.alt, .woocommerce-page #content input.button.alt {
				background: #c54200 !important;
				background: -webkit-gradient(linear,left top,left bottom,from(#fea817),to(#c54200)) !important;
				background: -webkit-linear-gradient(#fea817,#c54200) !important;
				background: -moz-linear-gradient(center top,#fea817 0%,#c54200 100%) !important;
				background: -moz-gradient(center top,#fea817 0%,#c54200 100%) !important;
				border-color: #a93c00 !important;
			}
			<?php } ?>
			<?php if (isset($data['action_buttons_radius']) && $data['action_buttons_radius'] != 30) { ?>
			.woocommerce a.button.alt, .woocommerce button.button.alt, .woocommerce input.button.alt, .woocommerce #respond input#submit.alt, .woocommerce #content input.button.alt, .woocommerce-page a.button.alt, .woocommerce-page button.button.alt, .woocommerce-page input.button.alt, .woocommerce-page #respond input#submit.alt, .woocommerce-page #content input.button.alt {
				border-radius: <?php echo @$data['action_buttons_radius'];?>px;
				-webkit-border-radius: <?php echo @$data['action_buttons_radius'];?>px;
				-moz-border-radius: <?php echo @$data['action_buttons_radius'];?>px;
			}
			<?php } else {?>
			.woocommerce a.button.alt, .woocommerce button.button.alt, .woocommerce input.button.alt, .woocommerce #respond input#submit.alt, .woocommerce #content input.button.alt, .woocommerce-page a.button.alt, .woocommerce-page button.button.alt, .woocommerce-page input.button.alt, .woocommerce-page #respond input#submit.alt, .woocommerce-page #content input.button.alt {
				border-radius: 3px;
				-webkit-border-radius: 3px;
				-moz-border-radius: 3px;
			}
			<?php } ?>
		</style>
	<?php
	}
}


add_action( 'after_setup_theme', 'justlanded_woocommerce_add_support');
if(!function_exists('justlanded_woocommerce_add_support')) {
	function justlanded_woocommerce_add_support() {
		add_theme_support('woocommerce');
	}
}

add_action( 'justlanded_before_head',             				 'justlanded_woocommerce_head');    // manipulate button settings with product links
add_action( 'justlanded_options_after_payment',   				 'justlanded_woocommerce_options'); // options panel additions
add_action( 'justlanded_options_after_page_content_textarea',    'justlanded_woocommerce_options_page_banner'); // options panel additions


//Unhook (remove) the WooCommerce sidebar on individual product pages
add_action('wp', create_function("", "if (is_archive(array('product'))) remove_action( 'woocommerce_sidebar', 'woocommerce_get_sidebar', 10);") );

//Unhook (remove) the WooCommerce sidebar on all pages
add_action('wp', create_function("", "if (is_singular(array('product'))) remove_action( 'woocommerce_sidebar', 'woocommerce_get_sidebar', 10);") );

remove_action( 'woocommerce_sidebar', 'woocommerce_get_sidebar', 10);

add_action('wp', 'justlanded_woocommerce_init');
if (!function_exists('justlanded_woocommerce_init')) {
	function justlanded_woocommerce_init() {

		$is_woocommerce = false; // we need to capture extensions etc. as well, so bult-in WC function not enough
		$body_classes = (array)get_body_class();

		if (in_array('woocommerce', $body_classes) || in_array('woocommerce-page', $body_classes))
			$is_woocommerce = true;

		if ($is_woocommerce == false) return;

		update_option( 'woocommerce_enable_lightbox', 'no', 99); // disable the lightbox, because we bring our own

		if( is_child_theme() ) {
			$temp_obj = wp_get_theme();
			$theme_obj = wp_get_theme( $temp_obj->get('Template') );
		} else {
			$theme_obj = wp_get_theme();
		}
		$theme_name = $theme_obj->get('Name');

		$wc_default_profile = intval(get_option($theme_name . '_global_justlanded_wc_profile'));
		if (!defined('SITE_DEFAULT_PROFILE_OVERRIDE') && $wc_default_profile && $wc_default_profile != 0) {
			define('SITE_DEFAULT_PROFILE_OVERRIDE', $wc_default_profile);
		}

		$show_titles  = intval(get_option($theme_name . '_global_justlanded_wc_titles'));
		if ($show_titles == 0) {
			add_filter('woocommerce_show_page_title', 'justlanded_woocommerce_hide_page_title');
		}

		$show_breadcrumb = intval(get_option($theme_name . '_global_justlanded_wc_breadcrumb'));
		if ($show_breadcrumb == 0) {
			remove_action( 'woocommerce_before_main_content', 'woocommerce_breadcrumb', 20, 0);
			add_filter('body_class','justlanded_woocommerce_add_no_breadcrumb_class');
		}
	}
}

add_action('init', 'justlanded_woocommerce_register_sidebar');
if (!function_exists('justlanded_woocommerce_register_sidebar'))
{
	function justlanded_woocommerce_register_sidebar() {
		register_sidebar(array(
			'name' => __('JustLanded WooCommerce Sidebar', 'justlanded'),
			'id' => 'justlanded-woocommerce-sidebar',
			'before_widget' => '<li id="%1$s" class="widget-container %2$s">',
			'after_widget' => "</li>",
			'before_title' => '<h3 class="widget-title">',
			'after_title' => '</h3>',
		));

	}
}

if (!function_exists('justlanded_woocommerce_get_sidebar'))
{
	function justlanded_woocommerce_get_sidebar() {
		?>
		<aside id="sidebar" class="three columns">
			<?php if ( is_active_sidebar('justlanded-woocommerce-sidebar') ) { ?>
				<div id="primary" class="widget-area">
					<ul class="sid">
						<?php dynamic_sidebar('justlanded-woocommerce-sidebar'); ?>
					</ul>
				</div>
			<?php } elseif (is_active_sidebar('primary-widget-area')) { ?>
				<div id="primary" class="widget-area">
					<ul class="sid">
						<?php dynamic_sidebar('primary-widget-area'); ?>
					</ul>
				</div>
			<?php } ?>
		</aside>
	<?php
	}
}

add_action( 'init', 'justlanded_woocommerce_clear_cart_url' );
function justlanded_woocommerce_clear_cart_url() {
	if ( isset( $_GET['empty-cart'] ) ) {
		global $woocommerce;
		$woocommerce->cart->empty_cart();
	}
}


function justlanded_woocommerce_output_content_wrapper() {} // dummy function because remove_action does not have the desired results
function justlanded_woocommerce_output_content_wrapper_end() {} // dummy function because remove_action does not have the desired results


add_action( 'after_setup_theme', 'justlanded_woocommerce_init_wrappers');
function justlanded_woocommerce_init_wrappers() {
	remove_action( 'woocommerce_before_main_content', 'woocommerce_output_content_wrapper', 10 );
	remove_action( 'woocommerce_after_main_content',  'woocommerce_output_content_wrapper_end', 10 );

	add_action('woocommerce_before_main_content', 'justlanded_woocommerce_output_content_wrapper', 10);
	add_action('woocommerce_after_main_content', 'justlanded_woocommerce_output_content_wrapper_end', 10);

	add_filter( 'template_include', 'justlanded_woocommerce_custom_template', 13 );
}

if (!function_exists('justlanded_woocommerce_custom_template'))
{
	function justlanded_woocommerce_custom_template( $template )
	{
		if( is_child_theme() ) {
			$temp_obj = wp_get_theme();
			$theme_obj = wp_get_theme( $temp_obj->get('Template') );
		} else {
			$theme_obj = wp_get_theme();
		}
		$theme_name = $theme_obj->get('Name');
		$empty_cart = intval(get_option($theme_name . '_global_justlanded_wc_empty_cart'));

		$show_sidebar = intval(get_option($theme_name . '_global_justlanded_wc_sidebar'));

		if (is_woocommerce()) {

			$tpl = basename($template);

			if ($tpl == 'taxonomy-product_cat.php' || $tpl == 'taxonomy-product_tag.php') $tpl = 'archive-product.php';

			if ($show_sidebar == 1) {  // determine whether to use sidebar templates
				$cur_template_file = plugin_dir_path( __FILE__ ) . DIRECTORY_SEPARATOR . 'templates/sidebar/' . DIRECTORY_SEPARATOR . $tpl;
				if(file_exists($cur_template_file))
					$template = $cur_template_file;
			} else {
				$cur_template_file = plugin_dir_path( __FILE__ ) . DIRECTORY_SEPARATOR . 'templates/no-sidebar/' . DIRECTORY_SEPARATOR . $tpl;
				if(file_exists($cur_template_file))
					$template = $cur_template_file;
			}

		}
		return $template;
	}
}


function justlanded_woocommerce_hide_page_title() {
	return false;
}


if (!function_exists('justlanded_woocommerce_add_no_breadcrumb_class'))
{
	function justlanded_woocommerce_add_no_breadcrumb_class($classes) {
		$classes[] = 'no-breadcrumb';
		return $classes;
	}
}

if (!function_exists('justlanded_woocommerce_add_nav_menu_meta_boxes')) {
	function justlanded_woocommerce_add_nav_menu_meta_boxes() {
		add_meta_box(
			'wl_login_nav_link',
			__( 'Shopping Cart', 'justlanded' ),
			'justlanded_woocommerce_nav_menu_link',
			'nav-menus',
			'side',
			'low'
		);
	}
}

if (!function_exists('justlanded_woocommerce_nav_menu_link')) {
	function justlanded_woocommerce_nav_menu_link() {
		?>
		<div id="posttype-wl-login" class="posttypediv">
			<div id="tabs-panel-woocommerce-cart" class="tabs-panel tabs-panel-active">
				<ul id="woocommerce-cart-checklist" class="categorychecklist form-no-clear">
					<li>
						<label class="menu-item-title">
							<input type="checkbox" class="menu-item-checkbox" name="menu-item[-1][menu-item-object-id]" value="-1"> <?php _e( 'Cart Icon w/ item count' ); ?>
						</label>
						<input type="hidden" class="menu-item-type" name="menu-item[-1][menu-item-type]" value="custom">
						<input type="hidden" class="menu-item-title" name="menu-item[-1][menu-item-title]" value="Cart ([#])">
						<input type="hidden" class="menu-item-url" name="menu-item[-1][menu-item-url]" value="<?php echo get_permalink( get_option( 'woocommerce_cart_page_id' ) ); ?>">
						<input type="hidden" class="menu-item-classes" name="menu-item[-1][menu-item-classes]" value="justlanded-woocommerce-cart">
					</li>
				</ul>
			</div>
			<p class="button-controls">
        			<span class="add-to-menu">
        				<input type="submit" class="button-secondary submit-add-to-menu right" value="Add to Menu" name="add-post-type-menu-item" id="submit-posttype-wl-login">
        				<span class="spinner"></span>
        			</span>
			</p>
		</div>
	<?php
	}
}
add_action('admin_init', 'justlanded_woocommerce_add_nav_menu_meta_boxes');



if (!function_exists('justlanded_woocommerce_modify_nav_menu_items')) {
	function justlanded_woocommerce_modify_nav_menu_items( $sorted_menu_items, $args = array() ) {
		global $woocommerce;
		$args = (array) $args;

		// Get menu id
		if ( $args['menu_id'] ) {
			$menu_id = $args['menu_id'];
		} elseif ( $args['theme_location'] ) {
			$menu_locations = get_nav_menu_locations();
			$menu_id        = $menu_locations[ $args['theme_location'] ];
		} else {
			$nav_item_db_id = $sorted_menu_items[1]->ID;
			$nav_menu       = wp_get_object_terms( $nav_item_db_id, 'nav_menu' );
			$menu_id        = $nav_menu[0]->term_id;
		}


		if ( $args['theme_location'] != "main-menu") return $sorted_menu_items;

		$count = $woocommerce->cart->cart_contents_count;

		$modified_nav_items = array();

		foreach ( $sorted_menu_items as $nav_item ) {
			$exclude = false;
			foreach ( $nav_item->classes as $nav_class) {
					if ( $nav_class == "justlanded-woocommerce-cart" ) {
						$nav_item->url = get_permalink( get_option( 'woocommerce_cart_page_id' ) );
						$title = sprintf(__('<span class="justlanded-woocommerce-cart-icon"></span> <span class="justlanded-woocommerce-cart-count">%d</span> items', 'justlanded'), $count);
						$nav_item->title = $title;
						break;
					}
			}
			if ($exclude == false) {
				$modified_nav_items[] = $nav_item;
			}
		}
		return $modified_nav_items;
	}
}
add_filter( 'wp_nav_menu_objects', 'justlanded_woocommerce_modify_nav_menu_items' , 90 , 2 );



add_filter('add_to_cart_fragments', 'justlanded_woocommerce_header_add_to_cart_fragment');
if (!function_exists('justlanded_woocommerce_header_add_to_cart_fragment')) {
	function justlanded_woocommerce_header_add_to_cart_fragment( $fragments ) {
		global $woocommerce;

		ob_start();

		?>
		<span class="justlanded-woocommerce-cart-count"><?php echo $woocommerce->cart->cart_contents_count; ?></span>
		<?php

		$fragments['span.justlanded-woocommerce-cart-count'] = ob_get_clean();

		return $fragments;
	}
}


?>