<?php
/**
Plugin Name: Dynamite by ShapingRain.com
Description: Dynamic content and variables for WordPress. A small but powerful tool for PPC marketers.
Author: ShapingRain.com Labs
Version: 1.1.1
Author URI: http://www.shapingrain.com
 */

global $shapingrain_dynamite_global_variables;
global $shapingrain_dynamite_global_sections;
global $shapingrain_dynamite_global_presetvars;


if (!function_exists('shapingrain_dynamite_init')) {
    function shapingrain_dynamite_init() {
        $options = get_option('shapingrain_dynamite_options');

        if (isset($options['content_filter']) && $options['content_filter'] == 1) {
            add_filter( 'the_content', 'shapingrain_dynamite_content_filter', 99);
        }

        if (isset($options['buffer_filter']) && $options['buffer_filter'] == 1) {
            add_action('wp_head',   'shapingrain_dynamite_buffer_start');
            add_action('wp_footer', 'shapingrain_dynamite_buffer_end');
        }
    }
}
add_action('init', 'shapingrain_dynamite_init', 1);

function shapingrain_dynamite_add_endpoints() {
    global $shapingrain_dynamite_global_sections;
    global $wp_rewrite;

    $shapingrain_dynamite_global_sections = new WP_Query( array( 'post_type' => 'dynamic_section', 'posts_per_page' => -1, 'meta_key' => 'srdyn_section_prio', 'order' => 'DESC', 'orderby' => 'meta_value_num' ) );
    $shapingrain_dynamite_global_sections->get_posts();

    if ($shapingrain_dynamite_global_sections->post_count > 0)
    {
        $this_vars = array();
        foreach ($shapingrain_dynamite_global_sections->posts as $section) {
            $m = get_post_meta( $section->ID );
            $slug = $m['srdyn_section_slug'][0];
            $conditions = $m['srdyn_conditions'][0];
            $no_match = false;
            foreach (explode("\n", $conditions) as $condition) {
                if (substr_count($condition, "=") > 0) {
                    list($cvar, $cval) = explode("=", $condition);
                }
                else {
                    $cvar = $condition;
                }
                $this_vars[] = $cvar;
            }
        }

        $this_vars = array_unique($this_vars);

        foreach ($this_vars as $cvar) {
            add_rewrite_endpoint( $cvar, EP_PERMALINK | EP_PAGES );
            if ('page' == get_option('show_on_front')) {
                $home_page_id = get_option('page_on_front');
                $post_data = get_post($home_page_id, ARRAY_A);
                $slug = $post_data['post_name'];
                $wp_rewrite->add_rule(  $cvar . '/([^/]+)/?',
                    'index.php?pagename='.$slug.'&'.$cvar.'=$matches[1]', 'top' );
            }
        }
    }
}
add_action('init', 'shapingrain_dynamite_add_endpoints');

function shapingrain_dynamite_custom_front_redirection($redirect_url, $requested_url){
    if( is_front_page() )
        return $requested_url;
    else
        return $redirect_url;
}
add_filter('redirect_canonical', 'shapingrain_dynamite_custom_front_redirection', 10, 2);


add_action('save_post', 'shapingrain_dynamite_save_section', 9999);
function shapingrain_dynamite_save_section($post_id) {
    $post = get_post($post_id);
    if ($post->post_type == 'dynamic_section') {
        shapingrain_dynamite_add_endpoints();
        flush_rewrite_rules();
    }
}

function shapingrain_dynamite_activate() {
    shapingrain_dynamite_add_endpoints();
    flush_rewrite_rules();
}
register_activation_hook( __FILE__, 'shapingrain_dynamite_activate' );

function shapingrain_dynamite_deactivate() {
    flush_rewrite_rules();
}
register_deactivation_hook( __FILE__, 'shapingrain_dynamite_deactivate' );


if (!is_admin())  add_action('wp', 'shapingrain_dynamite_process_sections', 99);
if (!function_exists('shapingrain_dynamite_process_sections')) {
    function shapingrain_dynamite_process_sections() {
        global $shapingrain_dynamite_global_variables;
        global $shapingrain_dynamite_global_sections;
        $options = get_option('shapingrain_dynamite_options');

        if ($shapingrain_dynamite_global_sections->post_count > 0)
        {

            foreach ($shapingrain_dynamite_global_sections->posts as $section) {
                $m = get_post_meta( $section->ID );
                $slug = $m['srdyn_section_slug'][0];
                $conditions = $m['srdyn_conditions'][0];
                $no_match = false;
                foreach (explode("\n", $conditions) as $condition) {
                    if (substr_count($condition, "=") > 0) {
                        list($cvar, $cval) = explode("=", $condition);
                    }
                    else {
                        $cvar = $condition;
                        $cval = "";
                    }
                    $cvar = strtolower(trim($cvar));
                    $cval = trim($cval);

                    if ($cval != shapingrain_dynamite_get_val($cvar)) $no_match = true;
                }

                $content_match = "";
                $content_no_match = "";
                $var_match = "";
                $var_no_match = "";

                if (isset($m['srdyn_output_if_conditions_met'])) $content_match    = $m['srdyn_output_if_conditions_met'][0];
                if (isset($m['srdyn_output_if_conditions_not_met'])) $content_no_match = $m['srdyn_output_if_conditions_not_met'][0];

                if (isset($m['srdyn_var_if_conditions_met'])) $var_match = $m['srdyn_var_if_conditions_met'][0];
                if (isset($m['srdyn_var_if_conditions_not_met'])) $var_no_match = $m['srdyn_var_if_conditions_not_met'][0];

                if ($no_match == false) {
                    if (!empty($content_match)) {
                        if (isset($options['shortcode_parse_content']) && $options['shortcode_parse_content'] == 1) $content_match = do_shortcode($content_match);
                        $shapingrain_dynamite_global_variables[$slug] = $content_match;
                    }
                    if ($var_match != "") {
                        $vars = shapingrain_dynamite_get_vars($var_match);
                    }
                }
                else {
                    if (!empty($content_no_match)) {
                        if (isset($options['shortcode_parse_content']) && $options['shortcode_parse_content'] == 1) $content_no_match = do_shortcode($content_no_match);
                        $shapingrain_dynamite_global_variables[$slug] = $content_no_match;
                    }
                    if ($var_no_match != "") {
                        $vars = shapingrain_dynamite_get_vars($var_no_match);
                    }
                }

                if (isset($vars) && is_array($vars)) {
                    foreach ($vars as $var => $val) {
                        $shapingrain_dynamite_global_variables[$var] = $val;
                    }
                }
            }
        }

        if (isset($shapingrain_dynamite_global_variables) && isset($shapingrain_dynamite_global_variables['profile'])) {
            if (!defined('SITE_DEFAULT_PROFILE_OVERRIDE')) define('SITE_DEFAULT_PROFILE_OVERRIDE', intval($shapingrain_dynamite_global_variables['profile']));
        }

        if (isset($shapingrain_dynamite_global_variables) && isset($shapingrain_dynamite_global_variables['wp_title'])) {
            if (!defined('SHAPINGRAIN_DYNAMITE_OVERRIDE_TITLE')) define('SHAPINGRAIN_DYNAMITE_OVERRIDE_TITLE', $shapingrain_dynamite_global_variables['wp_title']);
            add_filter( 'wp_title', 'justlanded_dynamite_wp_title', 9999999999, 2 );
        }
    }
}

function justlanded_dynamite_wp_title( $title, $sep ) {
    if (defined('SHAPINGRAIN_DYNAMITE_OVERRIDE_TITLE')) return SHAPINGRAIN_DYNAMITE_OVERRIDE_TITLE; else return $title;
}

add_action('template_redirect', 'shapingrain_dynamite_get_page_meta');
if (!function_exists('shapingrain_dynamite_get_page_meta')) {
    function shapingrain_dynamite_get_page_meta() {
        global $wp_query;
        global $shapingrain_dynamite_global_presetvars;
        if(isset($wp_query->post)) {
            $post_id = $wp_query->post->ID;
            $presetvars = get_post_meta($post_id, 'srdyn_section_presetvars', true);
            $vars = shapingrain_dynamite_get_vars($presetvars);
            if (isset($vars) && is_array($vars)) {
                foreach ($vars as $var => $val) {
                    $shapingrain_dynamite_global_presetvars[$var] = $val;
                }
            }
        }
        shapingrain_dynamite_process_sections();
    }
}

if (!function_exists('shapingrain_dynamite_get_vars')) {
    function shapingrain_dynamite_get_vars($var_string) {
        $options = get_option('shapingrain_dynamite_options');
        $var_string = trim($var_string);
        $tmp_vars = explode("\n", $var_string);
        $vars = array();
        if (is_array($tmp_vars)) {
            foreach ($tmp_vars as $tmp_var) {
                $tmp_var = trim($tmp_var);
                if (substr_count($tmp_var, "=")) {
                    $var_tmp_pos = strpos($tmp_var, "=");
                    $var[0] = substr($tmp_var, 0, $var_tmp_pos);
                    $var[1] = substr($tmp_var, $var_tmp_pos+1, strlen($tmp_var) - 1);
                    $set_var = trim($var[0]);
                    $set_val = trim($var[1]);
                    if (isset($options['shortcode_parse_vars']) && $options['shortcode_parse_vars'] == 1) $set_val = do_shortcode($set_val);
                    $vars[$set_var] = $set_val;
                }
            }
        }
        return $vars;
    }
}

if (!function_exists('shapingrain_dynamite_get_val')) {
    function shapingrain_dynamite_get_val($var, $check_cookie = false) {
        global $shapingrain_dynamite_global_presetvars;
        $val = "";
        $var = strtolower(str_replace(" ", "_", trim($var)));

        if (isset($shapingrain_dynamite_global_presetvars) && isset($shapingrain_dynamite_global_presetvars[$var])) {
            $val = $shapingrain_dynamite_global_presetvars[$var];
        }
        else
        {
            $q = get_query_var($var);
            if (isset($_COOKIE[$var]) && empty($q) && $check_cookie == true) {
                $val = $_COOKIE[$var];
            }
            elseif (!empty($q)) {
                $val = $q;
                setcookie($var, $val);
            }
        }
        return $val;
    }
}


add_action('admin_menu', 'shapingrain_dynamite_register_custom_menu_page');
if (!function_exists('shapingrain_dynamite_register_custom_menu_page')) {
    function shapingrain_dynamite_register_custom_menu_page() {
        add_submenu_page('edit.php?post_type=dynamic_section', 'settings', __('Settings', 'shapingrain_dynamite'), 'manage_options', 'shapingrain_dynamite_settings', 'shapingrain_dynamite_settings');
        add_submenu_page('edit.php?post_type=dynamic_section', 'support', __('Support', 'shapingrain_dynamite'), 'manage_options', 'shapingrain_dynamite_support', 'shapingrain_dynamite_settings');
    }
}

add_action('admin_init', 'shapingrain_dynamite_options_init' );
if (!function_exists('shapingrain_dynamite_options_init')) {
    function shapingrain_dynamite_options_init() {
        register_setting( 'shapingrain_dynamite_options', 'shapingrain_dynamite_options', 'shapingrain_dynamite_options_validate' );

        add_settings_section('filter_settings', __('Replacement Filter Settings'), 'shapingrain_dynamite_settings_section_filters', 'shapingrain_dynamite_settings');
        add_settings_field('content_filter', __('Enable the_content filter', 'shapingrain_dynamite'), 'shapingrain_dynamite_settings_wp_content_filter', 'shapingrain_dynamite_settings', 'filter_settings');
        add_settings_field('buffer_filter', __('Enable output buffering', 'shapingrain_dynamite'), 'shapingrain_dynamite_settings_buffer_filter', 'shapingrain_dynamite_settings', 'filter_settings');

        add_settings_section('shortcode_settings', __('Shortcode Settings'), 'shapingrain_dynamite_settings_section_shortcode', 'shapingrain_dynamite_settings');
        add_settings_field('shortcode_parse_vars', __('Parse shortcodes in replacement variables', 'shapingrain_dynamite'), 'shapingrain_dynamite_settings_shortcode_parse_vars', 'shapingrain_dynamite_settings', 'shortcode_settings');
        add_settings_field('shortcode_parse_content', __('Parse shortcodes in section content', 'shapingrain_dynamite'), 'shapingrain_dynamite_settings_shortcode_parse_content', 'shapingrain_dynamite_settings', 'shortcode_settings');
    }
}

function shapingrain_dynamite_options_validate($input) {
    return $input;
}

function shapingrain_dynamite_settings_section_filters() {
    echo '<p>' . __('These are optional content filters Dynamite will use to replace placeholders like <code>{variable}</code> with contents of variables or sections. These filters should only be activated if they are actually used.', 'shapingrain_dynamite'). '</p>';
}

function shapingrain_dynamite_settings_buffer_filter() {
    $options = get_option('shapingrain_dynamite_options');
    ?>
    <input name="shapingrain_dynamite_options[buffer_filter]" type="checkbox" value="1" <?php checked('1', shapingrain_dynamite_get_option($options, "buffer_filter"), true); ?> /> <em><?php _e('This will activate output buffering for variable replacements. Most likely to work with any theme in any place.', 'shapingrain_dynamite'); ?></em>
<?php
}

function shapingrain_dynamite_settings_wp_content_filter() {
    $options = get_option('shapingrain_dynamite_options');
    ?>
    <input name="shapingrain_dynamite_options[content_filter]" type="checkbox" value="1" <?php checked('1', shapingrain_dynamite_get_option($options, "content_filter"), true); ?> /> <em><?php _e('This will activate an additional the_content replacement filter. Should work in most cases.', 'shapingrain_dynamite'); ?></em>
<?php
}

function shapingrain_dynamite_settings_shortcode_parse_content() {
    $options = get_option('shapingrain_dynamite_options');
    ?>
    <input name="shapingrain_dynamite_options[shortcode_parse_content]" type="checkbox" value="1" <?php checked('1', shapingrain_dynamite_get_option($options, "shortcode_parse_content"), true); ?> /> <em><?php _e('If checked, shortcodes will be parsed in all section content fields.', 'shapingrain_dynamite'); ?></em>
<?php
}

function shapingrain_dynamite_settings_shortcode_parse_vars() {
    $options = get_option('shapingrain_dynamite_options');
    ?>
    <input name="shapingrain_dynamite_options[shortcode_parse_vars]" type="checkbox" value="1" <?php checked('1', shapingrain_dynamite_get_option($options, "shortcode_parse_vars"), true); ?> /> <em><?php _e('If checked, shortcodes will be parsed in replacement variables.', 'shapingrain_dynamite'); ?></em>
<?php
}


function shapingrain_dynamite_settings_section_shortcode() {
    echo '<p>' . __('Here you can define where you want content to be parsed for shortcodes.', 'shapingrain_dynamite'). '</p>';
}




function shapingrain_dynamite_get_option($options, $option) {
    if (isset($options[$option])) return $options[$option]; else return "";
}

if (!function_exists('shapingrain_dynamite_settings')) {
    function shapingrain_dynamite_settings() {
        $current = get_current_screen();
        $current_page = $current->id;
        ?>
        <div class="wrap">
            <div id="icon-themes" class="icon32"><br></div>
            <h2 class="nav-tab-wrapper">
                <a href="<?php menu_page_url( "shapingrain_dynamite_settings", true ); ?>" class="nav-tab<?php if ($current_page == "dynamic_section_page_shapingrain_dynamite_settings") echo ' nav-tab-active'; ?>"><?php echo __('Settings', 'shapingrain_dynamite'); ?></a>
                <a href="<?php menu_page_url( "shapingrain_dynamite_support", true ); ?>" class="nav-tab<?php if ($current_page == "dynamic_section_page_shapingrain_dynamite_support") echo ' nav-tab-active'; ?>"><?php echo __('Support', 'shapingrain_dynamite'); ?></a>
            </h2>
            <div class="clear"></div>
            <?php if ($current_page == "dynamic_section_page_shapingrain_dynamite_settings") { ?>
                <form method="post" action="options.php">
                    <?php settings_fields('shapingrain_dynamite_options'); ?>
                    <?php do_settings_sections('shapingrain_dynamite_settings'); ?>
                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
                    </p>
                </form>
            <?php } elseif ($current_page == "dynamic_section_page_shapingrain_dynamite_support" ) { ?>
                <h3><?php _e('Support', 'shapingrain_dynamite'); ?></h3>
                <p><?php _e('We provide free customer support for all our products via email to <code>support@shapingrain.com</code> and live support on <a href="http://www.shapingrain.com" target="_blank">shapingrain.com</a> and our public demo sites only.', 'shapingrain_dynamite'); ?></p>
                <p><?php _e('If you contact support, please provide a detailed description of your problem and provide as much additional information as possible.', 'shapingrain_dynamite'); ?></p>
                <p><?php _e('We may ask you for proof of purchase at any time, and personal information we obtain from you is stored securely and not made public or provided to third parties at any time.', 'shapingrain_dynamite'); ?></p>
            <?php } ?>
            <div class="clear"></div>
            <hr class="clear" style="border-bottom:1px solid #ccc; border-top:0;" />
            <p><?php _e('&copy; Copyright 2013 by <a href="http://www.shapingrain.com" target="_blank">ShapingRain.com</a> Labs', 'shapingrain_dynamite'); ?></p>
        </div>
    <?php
    }
}

if (!function_exists('shapingrain_dynamite_content_filter')) {
    function shapingrain_dynamite_content_filter($content) {
        global $shapingrain_dynamite_global_variables;
        $needles = array();
        $replacements = array();
        if (is_array($shapingrain_dynamite_global_variables) && count($shapingrain_dynamite_global_variables) > 0) {
            foreach ($shapingrain_dynamite_global_variables as $var => $val) {
                $needles[] = '{' . $var . "}";
                $replacements[] = $val;
            }
            $content = str_replace($needles, $replacements, $content);
        }
        return $content;
    }
}


if (!function_exists('shapingrain_dynamite_process_buffer')) {
    function shapingrain_dynamite_process_buffer($buffer) {
        $buffer = shapingrain_dynamite_content_filter($buffer);
        return $buffer;
    }
}
function shapingrain_dynamite_buffer_start() { ob_start("shapingrain_dynamite_process_buffer"); }
function shapingrain_dynamite_buffer_end()   { ob_end_flush(); }



add_action('admin_print_styles', 'shapingrain_dynamite_admin_css');
if (!function_exists('shapingrain_dynamite_admin_css')) {
    function shapingrain_dynamite_admin_css() {
        wp_register_style($handle = 'shapingrain_dynamite_admin_css', $src = plugins_url('css/shapingrain_dynamite.css', __FILE__), $deps = array(), $ver = '1.0.0', $media = 'all');
        wp_enqueue_style('shapingrain_dynamite_admin_css');
    }
}

add_action('wp_enqueue_scripts', 'shapingrain_dynamite_enqueue_custom_stylesheet');
if(!function_exists('shapingrain_dynamite_enqueue_custom_stylesheet')) {
    function shapingrain_dynamite_enqueue_custom_stylesheet() {
        //wp_register_style('shapingrain_dynamite_custom_stylesheet', plugin_dir_url("justlanded_woocommerce.php"). 'shapingrain-dynamite/shapingrain_dynamite.css', array(),'1.0.0', 'all');
        //wp_enqueue_style('shapingrain_dynamite_custom_stylesheet');
    }
}

if (!function_exists('shapingrain_dynamite_shortcode_dynamite_val')) {
    function shapingrain_dynamite_shortcode_dynamite_val( $attrs, $content = null ) {
        extract(shortcode_atts(array(
            'var' => '',
            'default' => '',
            'caps'  => 'false',
            'parse_shortcodes' => 'false',
            'set_cookie' => 'true',
            'uncompress' => 'false',
            'base64_decode' => 'false'
        ), $attrs));

        $out = "";

        if (isset($_GET[$var]) && trim($_GET[$var]) != "") $out = $_GET[$var]; else $out = $default;

        $cookie_name = str_replace(" ", "_", trim(strtolower($var)));
        if (isset($_COOKIE[$cookie_name]) && ($set_cookie == 'true' || $set_cookie == 1)) {
            if (!isset($_GET[$var])) $out = $_COOKIE[$cookie_name];
        }

        $out = trim($out);

        if ($uncompress == 'true' || $uncompress == 1) {
            try {
                @$out = gzinflate(base64_decode(strtr($out, '-_', '+/')));
            } catch (Exception $e) {
            }
        }

        if ($base64_decode == 'true' || $base64_decode == 1) {
            try {
                $out = base64_decode($out);
            } catch (Exception $e) {
            }
        }

        if ($caps == 'true' || $caps == 1) {
            $out = ucfirst($out);
        }

        if ($parse_shortcodes == 'true' || $parse_shortcodes == 1) {
            $out = do_shortcode($out);
        }

        if (($set_cookie == 'true' || $set_cookie == 1) && $out != "") {
            setcookie ($cookie_name, $out);
        }

        return $out;
    }
}
add_shortcode('dynamite_val', 'shapingrain_dynamite_shortcode_dynamite_val');

if (!function_exists('shapingrain_dynamite_shortcode_dynamite_content_condition')) {
    function shapingrain_dynamite_shortcode_dynamite_content_condition( $attrs, $content = null ) {
        global $shapingrain_dynamite_global_variables;
        $options = get_option('shapingrain_dynamite_options');
        extract(shortcode_atts(array(
            'var'	=> '',
            'val'   => '',
        ), $attrs));

        if (trim($var) == "" || trim($val) == "") return "";

        $var = trim(strtolower($var));
        $val = trim(strtolower($val));

        if (isset($options['shortcode_parse_content']) && $options['shortcode_parse_content'] == 1) {
            $content = do_shortcode(trim($content));
        }

        $getvar = get_query_var($var);
        if (!empty($getvar) && $getvar == $val) return $content;
        if (isset($_GET[$var]) && $_GET[$var] == $val) return $content;
        if (isset($shapingrain_dynamite_global_variables[$var]) && $shapingrain_dynamite_global_variables[$var] == $val) return $content;

        return "";
    }
}
add_shortcode('dynamite_condition', 'shapingrain_dynamite_shortcode_dynamite_content_condition');

if (!function_exists('shapingrain_dynamite_shortcode_dynamite_var')) {
    function shapingrain_dynamite_shortcode_dynamite_var( $attrs, $content = null ) {
        global $shapingrain_dynamite_global_variables;
        extract(shortcode_atts(array(
            'var'  	    => '',
            'default'   => '',
        ), $attrs));

        $var = trim(strtolower($var));
        $val = shapingrain_dynamite_get_val($var);

        if (empty($val)) {
            if (isset($shapingrain_dynamite_global_variables[$var])) {
                $val = $shapingrain_dynamite_global_variables[$var];
            }
        }

        if (empty($val)) {
            $val = $default;
        }

        return $val;
    }
}
add_shortcode('dynamite_var', 'shapingrain_dynamite_shortcode_dynamite_var');
add_shortcode('dynamite',     'shapingrain_dynamite_shortcode_dynamite_var');



if ( ! function_exists('shapingrain_dynamite_post_type') ) {
// Register Custom Post Type
    function shapingrain_dynamite_post_type() {
        $labels = array(
            'name'                => _x( 'Sections', 'Post Type General Name', 'shapingrain_dynamite' ),
            'singular_name'       => _x( 'Section', 'Post Type Singular Name', 'shapingrain_dynamite' ),
            'menu_name'           => __( 'Dynamite', 'shapingrain_dynamite' ),
            'parent_item_colon'   => __( 'Parent Section:', 'shapingrain_dynamite' ),
            'all_items'           => __( 'All Sections', 'shapingrain_dynamite' ),
            'view_item'           => __( 'View Section', 'shapingrain_dynamite' ),
            'add_new_item'        => __( 'Add New Section', 'shapingrain_dynamite' ),
            'add_new'             => __( 'New Section', 'shapingrain_dynamite' ),
            'edit_item'           => __( 'Edit Section', 'shapingrain_dynamite' ),
            'update_item'         => __( 'Update Section', 'shapingrain_dynamite' ),
            'search_items'        => __( 'Search sections', 'shapingrain_dynamite' ),
            'not_found'           => __( 'No sections found', 'shapingrain_dynamite' ),
            'not_found_in_trash'  => __( 'No sections found in Trash', 'shapingrain_dynamite' ),
        );

        $args = array(
            'label'               => __( 'dynamic_section', 'shapingrain_dynamite' ),
            'description'         => __( 'Dynamic content sections', 'shapingrain_dynamite' ),
            'labels'              => $labels,
            'supports'            => array( 'title' ),
            'hierarchical'        => false,
            'public'              => true,
            'show_ui'             => true,
            'show_in_menu'        => true,
            'show_in_nav_menus'   => false,
            'show_in_admin_bar'   => true,
            'menu_position'       => 20,
            'menu_icon'           => '',
            'can_export'          => true,
            'has_archive'         => false,
            'exclude_from_search' => true,
            'publicly_queryable'  => true,
            'rewrite'             => false,
            'capability_type'     => 'page',
        );
        register_post_type( 'dynamic_section', $args );
    }

// Hook into the 'init' action
    add_action( 'init', 'shapingrain_dynamite_post_type', 0 );
}

add_action('admin_init','shapingrain_dynamite_meta_init', 0);
if (!function_exists('shapingrain_dynamite_meta_init')) {
    function shapingrain_dynamite_meta_init() {
        if (is_admin()){
            require_once("classes/meta-box-class/my-meta-box-class.php");
            $prefix = 'srdyn_';

            /* Meta Box: General Settings */
            $config = array(
                'id' => $prefix . 'general',          // meta box id, unique per meta box
                'title' => __('Dynamic Section Settings', 'shapingrain_dynamite'),          // meta box title
                'pages' => array('dynamic_section'),      // post types, accept custom post types as well, default is array('post'); optional
                'context' => 'normal',            // where the meta box appear: normal (default), advanced, side; optional
                //'priority' => 'high',            // order of meta box: high (default), low; optional
                'fields' => array(),            // list of meta fields (can be added by field arrays)
                'local_images' => false,          // Use local or hosted images (meta box images for add/remove)
                'use_with_theme' => false          //change path if used with theme set to true, false for a plugin or anything else for a custom path(default false).
            );
            $meta_general =  new AT_Meta_Box($config);
            $meta_general->addText($prefix.'section_slug',array(
                    'name'=> __('Dynamic Section Slug', 'shapingrain_dynamite'),
                    'desc' => __('This is the section name you will query in order to output its contents. This field may not contain spaces and is limited to alphanumerical characters, dashes and underscores.', 'shapingrain_dynamite'))
            );
            $meta_general->addTextarea($prefix.'conditions',array(
                'name'=> __('Conditions', 'shapingrain_dynamite'),
                'desc' => __('This field contains conditions that need to be met; one condition per line, in format variable=value', 'shapingrain_dynamite')
            ));
            $meta_general->Finish();

            /* Meta Box: Content */
            $config = array(
                'id' => $prefix . 'content',          // meta box id, unique per meta box
                'title' => __('Dynamic Section Content', 'shapingrain_dynamite'),          // meta box title
                'pages' => array('dynamic_section'),      // post types, accept custom post types as well, default is array('post'); optional
                'context' => 'normal',            // where the meta box appear: normal (default), advanced, side; optional
                //'priority' => 'high',            // order of meta box: high (default), low; optional
                'fields' => array(),            // list of meta fields (can be added by field arrays)
                'local_images' => false,          // Use local or hosted images (meta box images for add/remove)
                'use_with_theme' => false          //change path if used with theme set to true, false for a plugin or anything else for a custom path(default false).
            );

            $meta_content =  new AT_Meta_Box($config);

            $meta_content->addParagraph($prefix.'intro_content', array(
                'value' => __("These WYSIWYG editor fields behave just like a normal page's content box. You can use shortcodes, add paragraphs, apply styles etc.", 'shapingrain_dynamite')
            ));

            $meta_content->addParagraph($prefix.'intro_content_2', array(
                'value' => __("The first field's content will be displayed if the conditions are met, while the second field's content will be displayed if the conditions aren't met. If the second field is empty, nothing will be returned if the condition is not met.", 'shapingrain_dynamite')
            ));

            $meta_content->addWysiwyg($prefix.'output_if_conditions_met',array(
                'name'=> __('Content if conditions are met', 'shapingrain_dynamite')
            ));

            $meta_content->addWysiwyg($prefix.'output_if_conditions_not_met',array(
                    'name'=> __('Content if conditions are not met', 'shapingrain_dynamite'))
            );
            $meta_content->Finish();

            /* Meta Box: Variable Assignments */
            $config = array(
                'id' => $prefix . 'variables',          // meta box id, unique per meta box
                'title' => __('Dynamic Section Variable Assignments', 'shapingrain_dynamite'),          // meta box title
                'pages' => array('dynamic_section'),      // post types, accept custom post types as well, default is array('post'); optional
                'context' => 'normal',            // where the meta box appear: normal (default), advanced, side; optional
                //'priority' => 'high',            // order of meta box: high (default), low; optional
                'fields' => array(),            // list of meta fields (can be added by field arrays)
                'local_images' => false,          // Use local or hosted images (meta box images for add/remove)
                'use_with_theme' => false          //change path if used with theme set to true, false for a plugin or anything else for a custom path(default false).
            );
            $meta_variables =  new AT_Meta_Box($config);

            $meta_variables->addParagraph($prefix.'intro_variables', array(
                'value' => __("In addition to displaying conditional content, you can also assign values to other variables.
                You can use this feature to assign button captions to a landing page, or to add custom tracking attributes to a script.", 'shapingrain_dynamite')
            ));

            $meta_variables->addTextarea($prefix.'var_if_conditions_met',array(
                'name'=> __('Variable Assignments if conditions are met', 'shapingrain_dynamite'),
                'desc' => __('One variable assignment per line, in format variable=value', 'shapingrain_dynamite')
            ));

            $meta_variables->addTextarea($prefix.'var_if_conditions_not_met',array(
                'name'=> __('Variable Assignments if conditions are not met', 'shapingrain_dynamite'),
                'desc' => __('One variable assignment per line, in format variable=value', 'shapingrain_dynamite')
            ));
            $meta_variables->Finish();

            /* Meta Box: Other */
            $config = array(
                'id' => $prefix . 'other',          // meta box id, unique per meta box
                'title' => __('Other Options', 'shapingrain_dynamite'),          // meta box title
                'pages' => array('dynamic_section'),      // post types, accept custom post types as well, default is array('post'); optional
                'context' => 'side',            // where the meta box appear: normal (default), advanced, side; optional
                'priority' => 'low',            // order of meta box: high (default), low; optional
                'fields' => array(),            // list of meta fields (can be added by field arrays)
                'local_images' => false,          // Use local or hosted images (meta box images for add/remove)
                'use_with_theme' => false          //change path if used with theme set to true, false for a plugin or anything else for a custom path(default false).
            );

            $meta_other =  new AT_Meta_Box($config);
            $meta_other->addNumber($prefix.'section_prio',array(
                    'name'=> __('Execution Priority', 'shapingrain_dynamite'),
                    'desc' => __('This defines in what order sections\' conditions are parsed. Numeric values only, the higher the priority, the earlier the section is parsed and executed.', 'shapingrain_dynamite'))
            );
            $meta_other->Finish();

            /* Meta Box: Pre-Set Variables for Pages */
            $config = array(
                'id' => $prefix . 'other',          // meta box id, unique per meta box
                'title' => __('Dynamite Options', 'shapingrain_dynamite'),          // meta box title
                'pages' => array('page'),      // post types, accept custom post types as well, default is array('post'); optional
                'context' => 'normal',            // where the meta box appear: normal (default), advanced, side; optional
                'priority' => 'high',            // order of meta box: high (default), low; optional
                'fields' => array(),            // list of meta fields (can be added by field arrays)
                'local_images' => false,          // Use local or hosted images (meta box images for add/remove)
                'use_with_theme' => false          //change path if used with theme set to true, false for a plugin or anything else for a custom path(default false).
            );

            $meta_presetvars =  new AT_Meta_Box($config);
            $meta_presetvars->addTextarea($prefix.'section_presetvars',array(
                    'name'=> __('Preset Variables', 'shapingrain_dynamite'),
                    'desc' => __('Use this field to pre-assign variables for this page only. Variables assigned here will be treated as if they were submitted as query parameters. One variable assignment per line, in format variable=value', 'shapingrain_dynamite'))
            );


            $meta_presetvars->Finish();



        }
    }
}


add_action( 'template_redirect', 'shapingrain_dynamite_preview_redirect', 1, 1);
function shapingrain_dynamite_preview_redirect() {
    global $wp_query;
    global $wp;


    if ( $wp_query->query_vars['post_type'] === 'dynamic_section' ) {
        $tpl = WP_PLUGIN_DIR . "/" . dirname(plugin_basename(__FILE__));
        include_once($tpl."/"."preview.php");
        exit();
    }

    return false;
}



?>